import java.util.*;
/**
 * Diese Klasse verarbeitet vom <code>rcssserver</code> eingehende Nachrichten
 * (z.T. unter Zuhilfenahme von Unterklassen, vgl. event()) und sorgt fr
 * die korrekte Speicherung relevanter Informationen.
 * Sie orientiert sich an der Klasse &#132;Actor&#147; des SimpleClient,
 * erweitert diese aber.
 *
 * @author Gruppe 11 "DieElf"
 * @author Ansprechpartner: <a href="mailto:rene@eDynamic.de">Ren Kubach</a>
 * @author <br>Ansprechpartner: <a href="mailto:fooxbox@web.de">Markus Fuchs</a>
 * @version $Id: EventControl.java,v 1.6 2002/07/07 20:56:55 mf Exp $
 */

public class EventControl
{
    /** Referenz zur ServerCommunication */
    private ServerCommunication serverCommunication;
	/** Referenz zum Player */
	private Player player;

    /**
     * Erzeugt eine neue Instanz der <code>EventControl</code>.
     *
     * @param serverCommunication vom Typ <code>ServerCommunication</code>,
     * dies ist die Verbindung zum rcssserver, ber die Nahrichten empfangen
     * werden.
     * @param player vom Typ <code>Player</code>, dies ist der Spieler, fr
     * den diese Klasse arbeitet
     *
     * @see ServerCommunication
     */
    public EventControl(Player player, ServerCommunication serverCommunication)
    {
        this.serverCommunication = serverCommunication;
    	this.player = player;
    }


     /**
     * Empfngt die Nachrichten des rcsservers. Diese werden automatisch nach
     * Typen (see, body_sense, usw.) unterschieden und an die richtigen Unterklassen
     * zum Parsen weitergegeben.<br>
     *
     * @return ein <code>Event</code>, das die geparsten Informationen aus der
     * empfangenen Nachricht enthlt.
     */
    public char receiveEvent()
    {
        String msg;
        char eventType = ' ';

        while (eventType == ' ') {
            // receive server message - block until one arrives
            msg = serverCommunication.receive();

			// init-Nachricht, wichtig fr Spielrichtung
			if(msg.startsWith("(init")) {
				parseInit(msg);
				eventType = 'i';
			}
            // see-Nachricht, enthlt alle visuellen Informationen
            else if (msg.startsWith("(see")) {
            	parseSee(msg);
				eventType = 's';
            }
            // sense_body-Nachricht, wichtig zum Auslsen von Aktionen
            else if (msg.startsWith("(sense_body")) {
            	parseBody(msg);
				eventType = 'b';
            }
            // hear-Nachricht, wichtig zum Herausfinden des PlayModes
            else if (msg.startsWith("(hear")) {
            	parseHear(msg);
				eventType = 'h';
            }
        }
        return eventType;
    }

	/**
	 * Parst die Informationen aus der <code>init</code>-Nachricht.<br>
	 * Reicht die geparsten Informationen eigenstndig an den
	 * <code>Player</code> weiter, damit der sie entsprechend verarbeiten kann.
	 *
	 * @param msg, ein <code>String</code>, der die <code>init</code>-Nachricht
	 * 		enthlt
	 */
	private void parseInit(String message) {
		try {
    		// zerlege Nachricht, vgl. hierzu VisualEvent
			StringTokenizer tokenizer = new StringTokenizer(message," ");
			// berspringe ersten Teilstring "(init"
			tokenizer.nextToken();
			// zweiter Teilstring enthlt Spielseite
			player.setMySide(tokenizer.nextToken().charAt(0));
			// dritter Teilstring enthlt Spielernummer
			player.setMyServerNr(Integer.valueOf(tokenizer.nextToken()).intValue());
			// vierter Teilstring enthlt PlayMode+")"
			String playMode = tokenizer.nextToken();
			// Klammer am Ende abschneiden
			player.setPlayMode(playMode.substring(0,playMode.length()-1));
			// Weltmodell des Spielers anlegen
			player.createWorldModel();
    	} catch (NoSuchElementException e) {
    	} catch (StringIndexOutOfBoundsException e) {
    	} catch (NumberFormatException e) {
    	}
	}
	
	/**
	 * Parst die Informationen aus einer <code>see</code>-Nachricht.
	 * Reicht hierzu die Nachricht an den <code>Player</code> weiter, damit
	 * dieser in einem eigenen Thread sein Weltmodell aktuelisieren kann.
	 *
	 * @param msg, ein <code>String</code>, der die <code>see</code>-Nachricht
	 * 		enthlt
	 */
	private void parseSee(String message) {
		player.updateWorldModel(message);
	}

	/**
	 * Parst die Informationen aus einer <code>sense_body</code>-Nachricht.<br>
	 * Reicht die geparsten Informationen eigenstndig an den
	 * <code>Player</code> weiter, damit der sie entsprechend verarbeiten kann.
	 *
	 * @param msg, ein <code>String</code>, der die
	 *		<code>sense_body</code>-Nachricht enthlt
	 */
	private void parseBody(String message) {
		try {
	    	// zerlege Nachricht, vgl. hierzu VisualEvent
			StringTokenizer tokenizer = new StringTokenizer(message," ");
			// berspringe ersten Teilstring "(sense_body"
			tokenizer.nextToken();
			// zweiter Teilstring enthlt Spielzeit
			player.setPlayTime(Integer.valueOf(tokenizer.nextToken()).intValue());
    	} catch (NoSuchElementException e) {
    	} catch (StringIndexOutOfBoundsException e) {
    	} catch (NumberFormatException e) {
    	}

	}

	
	/**
	 * Parst die Informationen aus einer <code>hear</code>-Nachricht.<br>
	 * Reicht die geparsten Informationen eigenstndig an den
	 * <code>Player</code> weiter, damit der sie entsprechend verarbeiten kann,
	 * bislang allerdings nur, wenn die Nachricht vom Schiedsrichter kommt.
	 *
	 * @param msg, ein <code>String</code>, der die <code>hear</code>-Nachricht
	 * 		enthlt
	 */
	private void parseHear(String message) {
		try {
	    	// zerlege Nachricht, vgl. hierzu VisualEvent
			StringTokenizer tokenizer = new StringTokenizer(message," ");
			// berspringe ersten Teilstring "(hear"
			tokenizer.nextToken();
			// zweiter Teilstring enthlt die Zeit, zu der die Nachricht gesprochen wurde
			// time = Integer.valueOf(tokenizer.nextToken()).intValue();
			tokenizer.nextToken();
			// dritter Teilstring enthlt Sprecher der Nachricht
			String sender = tokenizer.nextToken();
			// vierter Teilstring enthlt Nachricht+")"
			String hearMessage = tokenizer.nextToken();
			// Klammer am Ende abschneiden
			hearMessage = hearMessage.substring(0,hearMessage.length()-1);
			// Falls die Nachricht vom Schiedsrichter stammt,
			// ndere den Spielmodus:
			if(sender.equals("referee"))
				player.setPlayMode(hearMessage);
    	} catch (NoSuchElementException e) {
    	} catch (StringIndexOutOfBoundsException e) {
    	} catch (NumberFormatException e) {
    	}
	}
	
}